// Copyright (c) 2006 Atmel Corporation
// $Id: adc.h,v 1.2 2007/05/01 17:55:22 eweddington Exp $


#ifndef ADC_H
#define ADC_H

#include <avr/io.h>
#include <stdbool.h>
#include <stdint.h>


#define ADC_MUX_BITS        (uint8_t)((1 << MUX0)| (1 << MUX1) | (1 << MUX2) | (1 << MUX3) | (1 << MUX4))

#define ADC_CHANNEL_0           0
#define ADC_CHANNEL_1           1
#define ADC_CHANNEL_2           2
#define ADC_CHANNEL_3           3
#define ADC_CHANNEL_4           4
#define ADC_CHANNEL_5           5
#define ADC_CHANNEL_6           6
#define ADC_CHANNEL_7           7
#define ADC_CHANNEL_00_10X      8
#define ADC_CHANNEL_10_10X      9
#define ADC_CHANNEL_00_200X     10
#define ADC_CHANNEL_10_200X     11
#define ADC_CHANNEL_22_10X      12
#define ADC_CHANNEL_32_10X      13
#define ADC_CHANNEL_22_200X     14
#define ADC_CHANNEL_32_200X     15
#define ADC_CHANNEL_01          16
#define ADC_CHANNEL_11          17
#define ADC_CHANNEL_21          18
#define ADC_CHANNEL_31          19
#define ADC_CHANNEL_41          20
#define ADC_CHANNEL_51          21
#define ADC_CHANNEL_61          22
#define ADC_CHANNEL_71          23
#define ADC_CHANNEL_02          24
#define ADC_CHANNEL_12          25
#define ADC_CHANNEL_22          26
#define ADC_CHANNEL_32          27
#define ADC_CHANNEL_42          28
#define ADC_CHANNEL_52          29
#define ADC_CHANNEL_1220mV      30
#define ADC_CHANNEL_GROUND      31


// For ATmega128
#define ADC_PORT                PORTF
#define ADC_PORT_DIRECTION      DDRF
#define ADC_PIN                 PINF

#define ADC_REFERENCE_VOLTAGE   (2500)      // milliVolts units
#define ADC_STEPS_MAX           (1024)



#define ADPS_BITS   (uint8_t)((1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0))

#define ADC_CLOCK_DIV2      1
#define ADC_CLOCK_DIV4      2
#define ADC_CLOCK_DIV8      3
#define ADC_CLOCK_DIV16     4
#define ADC_CLOCK_DIV32     5
#define ADC_CLOCK_DIV64     6
#define ADC_CLOCK_DIV128    7



// API
void adc_init(void);

#define adc_conversion_start()      (ADCSR |= (uint8_t)(1 << ADSC))
#define adc_complete()              (ADCSR & (uint8_t)(1 << ADIF))
#define adc_complete_clear()        (ADCSR |= (uint8_t)(1 << ADIF))

#define adc_enable()                (ADCSR |= (uint8_t)(1 << ADEN))
#define adc_disable()               (ADCSR &= (uint8_t)~(1 << ADEN))

#define adc_reference_internal()    (ADMUX |= (uint8_t)((1 << REFS1) | (1 << REFS0)))
#define adc_reference_external()    (ADMUX &= (uint8_t)~((1 << REFS1) | (1 << REFS0)))
#define adc_reference_power()       (ADMUX = ((ADMUX & (uint8_t)~(1 << REFS1)) | (uint8_t)(1 << REFS0)))

#define adc_left_shift_result()     (ADMUX |= (uint8_t)(1 << ADLAR))
#define adc_right_shift_result()    (ADMUX &= (uint8_t)~(1 << ADLAR))

#define adc_mode_freerun()          (ADCSR |= (uint8_t)(1 << ADFR))

#define adc_mode_single()           (ADCSR &= (uint8_t)~(1 << ADFR))

bool adc_prescaler_set(const uint8_t prescaler);

bool adc_channel(const uint8_t channel);

uint16_t adc_read(void);

uint16_t adc_read_avg(uint8_t samples);

#define adc_millivolts2level(x)     ((ADC_STEPS_MAX * (uint32_t)x) / ADC_REFERENCE_VOLTAGE)

#define adc_level2millivolts(x)     ((uint16_t)((ADC_REFERENCE_VOLTAGE * (uint32_t)x) / ADC_STEPS_MAX))

#endif
