// Copyright (c) 2006 Atmel Corporation
// $Id: ring.h,v 1.6 2007/05/01 17:55:22 eweddington Exp $

/**
*   @file
    Ring Buffer Header File
    Version 1.1
    Updated 2006/08/25 by E. Weddington

*   \brief Declarations for the Ring Buffer functionality
*
*/

#ifndef RING_H
#define RING_H

#include <stdint.h>
#include <stdbool.h>

typedef uint16_t ring_index_t;
typedef uint8_t *ring_buffer_t;
typedef uint8_t ring_count_t;
typedef uint8_t ring_size_t;
typedef struct
{
    ring_buffer_t buffer;
    ring_size_t size;
    ring_index_t head_index;
    ring_index_t tail_index;
    ring_count_t count;
} ring_t;

/** This function initializes the ring buffer. */
bool ring_init(ring_t *r, ring_buffer_t buffer, ring_size_t size);

/** This function can clear, or reset, the ring buffer. */
bool ring_clear(ring_t *r);

/** This function will add data to the ring buffer. */
bool ring_add(ring_t *r, uint8_t data);

/** This function will remove data from the ring buffer. */
uint8_t ring_remove(ring_t *r);

/** This function can peek ahead to the data in the ring buffer. */
uint8_t ring_peek(ring_t *r, uint8_t *buf, uint8_t count);

/** This function returns a highmark value during flow contol. */
bool ring_above_highmark(ring_t *r);

/** This function returns a lowmark value during flow contol. */
bool ring_below_lowmark(ring_t *r);

/** ring buffer counter initialization */
#define ring_free_count(r)      (((r)->size) - ((r)->count))

#endif
