// Copyright (c) 2006 Atmel Corporation
// $Id: ring_add.c,v 1.6 2007/05/01 17:55:22 eweddington Exp $

/**
*   @file
    Ring Buffer Add File
    Version 1.1
    Updated 2006/08/25 by E. Weddington

*   \brief Creates operation for adding data to the ring buffer
*
*/

#define RING_ADD_C

#include <stdbool.h>
#include <stdint.h>
#include "ring.h"

/** This boolean will reference the ring_t structure and add data
    to the ring buffer as it increments the index. If data
    can be added to the ring buffer, the boolean will then return
    a true. If no addition was done, then a false will be
    returned.
    
    NOTE: Data added will be one value less than the size
    of the defined ring buffer. An example of this is a buffer
    size of [10] will only allow 9 values to be added into the buffer*/
bool ring_add(ring_t *r, uint8_t data)
{
    // Get the next head index.
    ring_index_t temp_head = r->head_index + 1;
    if(r->head_index >= r->size - 1)
    {
        temp_head = 0;
    }

    if(temp_head != r->tail_index)
    {
        r->buffer[r->head_index] = data;    // Buffer receives data
        r->head_index = temp_head;
        r->count++;
        return(true);       // Return complete
    }

    return(false);          // Return incomplete
}
