// Copyright (c) 2006 Atmel Corporation
// $Id: ring_init.c,v 1.8 2007/05/01 17:55:22 eweddington Exp $

/**
*   @file
    Ring Buffer Initialization File
    Version 1.1
    Updated 2006/08/25 by E. Weddington

*   \brief Initializes the ring buffer for proper operation
*
*/

#define RING_INIT_C

#include <stdbool.h>
#include <stdlib.h>
#include "ring.h"

/** Using the ring_init file sets the index pointers to their default
    location. The clear command is also called to set 0 values within
    the ring buffer functions. 

    The \param size is the actual size of \param buffer and must be equal
    to or greater than 2. The actual amount of storage available in the ring
    buffer is \param size - 1.
    
    todo: describe return value
*/
bool ring_init(ring_t *r, ring_buffer_t buffer, ring_size_t size)
{
    bool success = false;
    bool clear;
    if (size >= 2 && r != NULL && buffer != NULL)
    {
        r->buffer = buffer;     // Set index pointers
        r->size = size;
        clear = ring_clear(r);          // Call ring_clear function
        if(clear)
        {
                success = true;
        }
    }
    return(success);
}
