// Copyright (c) 2006 Atmel Corporation
// $Id: ring_peek.c,v 1.6 2007/05/01 17:55:22 eweddington Exp $

/**
*   @file
    Ring Buffer Peek File
    Version 1.1
    Updated 2006/08/25 by E. Weddington

*   \brief Allows for a quick peek to the data ahead in the
*          ring buffer.
*
*/

#define RING_PEEK_C

#include <stdint.h>
#include <stdlib.h>
#include "ring.h"


/** To peek ahead, index pointing with structure referncing is used.
    A new buffer is called to store the temporary data. Once the
    buffer is read, it is cleared out. The count parameter should not be
    larger than the buffer size pointed to by buf.
*/
uint8_t ring_peek(ring_t *r, uint8_t *buf, uint8_t count)
{
    ring_index_t index;
    uint8_t total = 0;

    if (r != NULL && buf != NULL)
    {
        index = r->tail_index;
        // Compare the head index to the tail index
        while(count && index != r->head_index)
        {
            *buf++ = r->buffer[index++];    // Grab data requested and increment
    
            if(index >= r->size)        // End of buffer determination
            {
                index = 0;
            }
    
            count--;
            total++;
        }
    }
    return(total);
}
