// Copyright (c) 2006 Atmel Corporation
// $Id: ring_remove.c,v 1.5 2007/05/01 17:55:22 eweddington Exp $

/**
*   @file
    Ring Buffer Remove File
    Version 1.1
    Updated 2006/08/25 by E. Weddington

*   \brief Creates operation for removing data from the ring buffer
*
*/

#define RING_REMOVE_C

#include <stdint.h>
#include "ring.h"

/** Data is retreived from the ring buffer and assigned 'data'. Upon
    retrieval, the ring buffer index is decremented. */
uint8_t ring_remove(ring_t *r)
{
    uint8_t data = 0;

    // Compare the head index and the tail index
    if(r->head_index != r->tail_index)
    {
        // Get the Data from the buffer.
        data = r->buffer[r->tail_index];

        // Get the next tail index.
        ring_index_t next_tail = r->tail_index + 1;
        if(r->tail_index >= r->size - 1)
        {
            next_tail = 0;
        }

        r->tail_index = next_tail;      // Move index
        r->count--;             // Decrement count
    }

    return(data);
}
