// Copyright (c) 2006 Atmel Corporation
// $Id: swspi_master.c,v 1.4 2007/05/01 17:55:22 eweddington Exp $

#define SWSPI_MASTER_C

// Includes
#include <stdint.h>
#include "swspi.h"

// Static Variables
static volatile uint8_t *spi_port;
static volatile uint8_t *spi_port_dir;
static uint8_t spi_mosi_bit;
static uint8_t spi_miso_bit;
static uint8_t spi_sck_bit;


// Macros
#define spi_sck_hi()    *spi_port |= (uint8_t)(1 << spi_sck_bit)
#define spi_sck_lo()    *spi_port &= (uint8_t)~(1 << spi_sck_bit)

#define spi_mosi_hi()   *spi_port |= (uint8_t)(1 << spi_mosi_bit)
#define spi_mosi_lo()   *spi_port &= (uint8_t)~(1 << spi_mosi_bit)

#define spi_miso_read() *spi_port & (uint8_t)(1 << spi_miso_bit)


// Implementation

void swspi_master_init(volatile uint8_t *port, 
    volatile uint8_t *port_dir, uint8_t mosi_bit, uint8_t miso_bit,
    uint8_t sck_bit)
{
    spi_port = port;
    spi_port_dir = port_dir;
    spi_mosi_bit = mosi_bit;
    spi_miso_bit = miso_bit;
    spi_sck_bit = sck_bit;

    // Output lines.
    spi_sck_lo();
    spi_mosi_lo();
    *spi_port_dir |= (uint8_t)(1 << spi_mosi_bit) | (1 << spi_sck_bit);
    
    // Input lines.
    *spi_port_dir &= (uint8_t)~(1 << spi_miso_bit);
    
    return;
}



// Mode 0 transport.
uint8_t swspi_transmit_byte(const uint8_t data_out)
{
    uint8_t data_in = 0x00;
    
    for(uint8_t mask = 0x80; mask; mask >>= 1)
    {
        // Data out pin.
        if (data_out & mask)
        {
            spi_mosi_hi();
        }
        else
        {
            spi_mosi_lo();
        }
        
        // Clock hi.
        spi_sck_hi();

        // Clock lo.
        spi_sck_lo();

        // Read data in.
        if (spi_miso_read())
        {
            data_in |= mask;
        }
        else
        {
            data_in &= (uint8_t)~mask;
        }
    }
    
    return(data_in);
}


uint8_t swspi_receive_byte(void)
{
    return(swspi_transmit_byte(SPI_DUMMY_BYTE));
}
